<?php
/**
 * @package    JDiDEAL
 *
 * @author     Roland Dalmulder <contact@rolandd.com>
 * @copyright  Copyright (C) 2009 - 2023 RolandD Cyber Produksi. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link       https://rolandd.com
 */

namespace Jdideal\Addons;

defined('_JEXEC') or die;

use JDatabaseDriver;
use Joomla\CMS\Factory;
use RuntimeException;
use stdClass;

/**
 * JoomDonation addon.
 *
 * @package  JDiDEAL
 * @since    2.13.0
 */
class AddonJoomdonation implements AddonInterface
{
	/**
	 * Database interface
	 *
	 * @var    JDatabaseDriver
	 * @since  5.3.0
	 */
	private $db;

	/**
	 * Constructor.
	 *
	 * @since  5.3.0
	 */
	public function __construct()
	{
		$this->db = Factory::getDbo();
	}

	/**
	 * Get the name of the addon.
	 *
	 * @return  string  The name of the component.
	 *
	 * @since   5.3.0
	 */
	public function getName(): string
	{
		return 'Joom Donation';
	}

	/**
	 * Collect the order information.
	 *
	 * @param   string  $orderId  The ID to get the order data for
	 * @param   array   $data     Array with payment info
	 *
	 * @return  array  Array with order information.
	 *
	 * @since   2.13.0
	 *
	 * @throws  RuntimeException
	 */
	public function getOrderInformation(string $orderId, array $data): array
	{
		$query = $this->db->getQuery(true)
			->select(
				$this->db->quoteName(
					[
						'email',
						'published',
						'amount'
					]
				)
			)
			->from($this->db->quoteName('#__jd_donors'))
			->where($this->db->quoteName('id') . ' = ' . $this->db->quote($orderId));
		$this->db->setQuery($query);
		$order = $this->db->loadObject();

		if (!$order)
		{
			$data['order_status'] = false;

			return $this->callBack($data);
		}

		$data['order_total']  = $order->amount;
		$data['order_status'] = $this->translateOrderStatus($order->published);
		$data['user_email']   = $order->email;

		// Return the data
		return $data;
	}

	/**
	 * Perform the callback for any additional tasks to be done.
	 *
	 * @param   array  $data  An array of order data.
	 *
	 * @return  array  An array of order data.
	 *
	 * @since   2.13.0
	 */
	public function callBack(array $data): array
	{
		return $data;
	}

	/**
	 * Translate component order status to RO Payments status.
	 *
	 * @param   string  $orderStatus  The component order status.
	 *
	 * @return  string  The RO Payments order status.
	 *
	 * @since   2.13.0
	 */
	public function translateOrderStatus(string $orderStatus): string
	{
		switch ($orderStatus)
		{
			case '1':
				return 'C';
				break;
			default:
				return 'P';
				break;
		}
	}

	/**
	 * Collect customer information.
	 *
	 * @param   string  $orderId  The ID of the order to get the info from.
	 *
	 * @return  array  The array of customer information.
	 *
	 * @since   2.13.0
	 */
	public function getCustomerInformation(string $orderId): array
	{
		return [];
	}

	/**
	 * Get the component order status name.
	 *
	 * @param   array  $data  The name of the RO Payments order status.
	 *
	 * @return  string  The name of the component order status.
	 *
	 * @since   2.13.0
	 */
	public function getOrderStatusName(array $data): string
	{
		switch ($data['order_status'])
		{
			case 'C':
				return '1';
				break;
			default:
				return '0';
				break;
		}
	}

	/**
	 * Get the link to the component.
	 *
	 * @return  string  The link to the component.
	 *
	 * @since   2.13.0
	 */
	public function getComponentLink(): string
	{
		return 'index.php?option=com_jdonation';
	}

	/**
	 * Get the link to the order on the administrator side.
	 *
	 * @param   string  $orderId  The order ID to get the link for.
	 *
	 * @return  string  The link to the order.
	 *
	 * @since   2.13.0
	 */
	public function getAdminOrderLink(string $orderId): string
	{
		return 'index.php?option=com_jdonation&task=donor.edit&id[]=' . $orderId;
	}

	/**
	 * Get the link to the order on the frontend.
	 *
	 * @param   string  $orderId      The order ID for the link
	 * @param   string  $orderNumber  The order number for the link
	 *
	 * @return  string  The link to the order.
	 *
	 * @since   2.13.0
	 */
	public function getOrderLink(string $orderId, string $orderNumber): string
	{
		return 'index.php?option=com_jdonation&view=history';
	}

	/**
	 * Replace addon specific placeholders.
	 *
	 * @param   stdClass  $details  The transaction details
	 * @param   string    $message  The message to replace the placeholders for
	 *
	 * @return  string  The message with replaced placeholders.
	 *
	 * @since   5.3.0
	 */
	public function replacePlaceholders(stdClass $details, string $message): string
	{
		return $message;
	}
}
