<?php
/**
 * @package    JDiDEAL
 *
 * @author     Roland Dalmulder <contact@rolandd.com>
 * @copyright  Copyright (C) 2009 - 2023 RolandD Cyber Produksi. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link       https://rolandd.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Language\Text;

/**
 * Load the RO Payments installer.
 *
 * @return  void.
 *
 * @since  3.3.0
 */
class PlgsystemjdidealjoomdonationInstallerScript
{
	/**
	 * Run the preflight operations.
	 *
	 * @param   object  $parent  The parent class.
	 *
	 * @return  boolean  True on success | False on failure.
	 *
	 * @since   4.7.0
	 *
	 * @throws  Exception
	 */
	public function preflight($parent): bool
	{
		// Check if RO Payments is installed
		if ($parent === 'install' && ComponentHelper::isInstalled('com_jdidealgateway') === 0)
		{
			Factory::getApplication()->enqueueMessage(
				Text::_('PLG_JOOMDONATION_JDIDEAL_JDIDEALGATEWAY_NOT_INSTALLED'),
				'error'
			);

			return false;
		}

		// Check if extension is installed
		if ($parent === 'install' && ComponentHelper::isInstalled('com_jdonation') === 0)
		{
			Factory::getApplication()->enqueueMessage(
				Text::_('PLG_JOOMBOOKING_JDIDEAL_JOOMBOOKING_NOT_INSTALLED'),
				'error'
			);

			return false;
		}

		return true;
	}

	/**
	 * Run the postflight operations.
	 *
	 * @param   object  $parent  The parent class.
	 *
	 * @return  boolean True on success | False on failure.
	 *
	 * @since   3.3.0
	 *
	 * @throws  Exception
	 */
	public function postflight($parent): bool
	{
		// Copy the files to the JoomDonation folder
		$src  = JPATH_SITE . '/plugins/system/jdidealjoomdonation/files/';
		$dest = JPATH_SITE . '/components/com_jdonation/payments';

		if (Folder::copy($src, $dest, '', true))
		{
			// Add the database entry
			$db       = Factory::getDbo();
			$row      = new stdClass;
			$pluginId = $this->getPluginId();

			// Get the largest ordering value for a given where clause.
			$query = $db->getQuery(true)
				->select('MAX(' . $db->quoteName('ordering') . ')')
				->from($db->quoteName('#__jd_payment_plugins'))
				->where($db->quoteName('published') . ' = 1');
			$db->setQuery($query);
			$row->ordering  = (int) $db->loadResult() + 1;
			$row->published = 0;

			$row->name          = 'os_jdideal';
			$row->title         = 'RO Payments';
			$row->author        = 'RolandD Cyber Produksi';
			$row->creation_date = '20 June 2023';
			$row->copyright     = 'Copyright (C) 2009 - 2023 RolandD Cyber Produksi. All rights reserved.';
			$row->license       = 'GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html';
			$row->author_email  = 'contact@rolandd.com';
			$row->author_url    = 'https://rolandd.com';
			$row->version       = '8.3.0-dev';
			$row->description   = 'Payment plugin for RO Payments';

			if ($pluginId)
			{
				$row->id = $pluginId;
				$db->updateObject('#__jd_payment_plugins', $row, 'id');
			}
			else
			{
				$db->insertObject('#__jd_payment_plugins', $row);
			}
		}

		// Enable the plugin
		$app   = Factory::getApplication();
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
			->update($db->quoteName('#__extensions'))
			->set($db->quoteName('enabled') . ' =  1')
			->where($db->quoteName('element') . ' = ' . $db->quote('jdidealjoomdonation'))
			->where($db->quoteName('type') . ' = ' . $db->quote('plugin'));
		$db->setQuery($query);

		if (!$db->execute())
		{
			$app->enqueueMessage(
				Text::sprintf('PLG_JOOMDONATION_PAYMENT_JDIDEAL_PLUGIN_NOT_ENABLED', $db->getErrorMsg()),
				'error'
			);

			return false;
		}

		$app->enqueueMessage(Text::_('PLG_JOOMDONATION_PAYMENT_JDIDEAL_PLUGIN_ENABLED'));

		return true;
	}

	/**
	 * Get the plugin ID.
	 *
	 * @return  integer  The ID of the plugin.
	 *
	 * @since   3.3.0
	 */
	private function getPluginId(): int
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
			->select($db->quoteName('id'))
			->from($db->quoteName('#__jd_payment_plugins'))
			->where($db->quoteName('name') . ' = ' . $db->quote('os_jdideal'));
		$db->setQuery($query);

		return (int) $db->loadResult();
	}

	/**
	 * Uninstall the plugin.
	 *
	 * @param   object  $parent  The parent class calling.
	 *
	 * @return  void
	 *
	 * @since   3.3.0
	 */
	public function uninstall($parent): void
	{
		// Remove the files from the Joom Donation folder
		$paths = [
			JPATH_SITE . '/components/com_jdonation/payments/os_jdideal.php',
			JPATH_SITE . '/components/com_jdonation/payments/os_jdideal.xml',
			JPATH_ADMINISTRATOR . '/components/com_jdidealgateway/models/addons/joomdonation.php'
		];

		foreach ($paths as $path)
		{
			File::delete($path);
		}

		// Remove the database entry
		$pluginId = $this->getPluginId();

		if ($pluginId)
		{
			$db    = Factory::getDbo();
			$query = $db->getQuery(true)
				->delete($db->quoteName('#__jd_payment_plugins'))
				->where($db->quoteName('id') . ' = ' . $pluginId);
			$db->setQuery($query)->execute();
		}
	}
}
